#include "version.h"

#if defined(USE_STARSKY) || defined(USE_RAIN) || defined(USE_SNOW)

#include "wl_def.h"
#include "wl_shade.h"

#if defined(USE_RAIN) || defined(USE_SNOW)
    uint32_t rainpos = 0;
#endif

byte mergetable[16][256];

typedef struct {
    int32_t x, y, z;
} point3d_t;

#define MAXPOINTS 2000
point3d_t points[MAXPOINTS];

byte moon[100]={
     0,  0, 97, 88, 85, 86, 89, 99,  0,  0,
     0, 92, 86, 85, 85, 86, 86, 88, 94,  0,
    97, 87, 85, 87, 86, 86, 87, 87, 88, 99,
    88, 85, 85, 85, 86, 86, 87, 87, 88, 90,
    86, 85, 85, 86, 86, 87, 87, 88, 89, 91,
    86, 85, 87, 90, 88, 87, 88, 88, 90, 92,
    89, 86, 88, 89, 87, 87, 88, 89, 92, 94,
    98, 89, 87, 87, 87, 88, 89, 91, 95, 91,
     0, 93, 88, 89, 88, 90, 92, 94, 98,  0,
     0,  0, 98, 91, 90, 92, 98, 90,  0,  0 };

void Init3DPoints()
{
    int hvheight = viewheight >> 1;
    for(int i = 0; i < MAXPOINTS; i++)
    {
        point3d_t *pt = &points[i];
        pt->x = 16384 - (rand() & 32767);
        pt->z = 16384 - (rand() & 32767);
        float len = sqrt((float)pt->x * pt->x + (float)pt->z * pt->z);
        int j=50;
        do
        {
            pt->y = 1024 + (rand() & 8191);
            j--;
        }
        while(j > 0 && (float)pt->y * 256.F / len >= hvheight);
    }
#ifdef TRANSPARENTRAIN
    for(int shade = 0; shade < 16; shade++)
        for (int col = 0; col < 256; col++)
            mergetable[shade][col] = MergeColour(int(shade*1.5) + 77, col, 2 - (shade * 0.1), 1);
#endif
}

#endif

#ifdef USE_STARSKY

void DrawStarSky(byte *vbuf, uint32_t vbufPitch)
{
    int hvheight = viewheight >> 1;
    int hvwidth = viewwidth >> 1;

    byte *ptr = vbuf;
    for(int i = 0; i < hvheight; i++, ptr += vbufPitch)
        memset(ptr, 0, viewwidth);

    for(int i = 0; i < MAXPOINTS; i++)
    {
        point3d_t *pt = &points[i];
        int32_t x = pt->x * viewcos + pt->z * viewsin;
        int32_t y = pt->y << 16;
        int32_t z = (pt->z * viewcos - pt->x * viewsin) >> 8;
        if(z <= 0) continue;
        int shade = z >> 18;
        if(shade > 15) continue;
        int32_t xx = x / z + hvwidth;
        int32_t yy = hvheight - y / z;
        if(xx >= 0 && xx < viewwidth && yy >= 0 && yy < hvheight)
            vbuf[yy * vbufPitch + xx] = shade + 85;
    }

/*    int32_t x = 16384 * viewcos + 16384 * viewsin;
    int32_t z = (16384 * viewcos - 16384 * viewsin) >> 8;
    if(z <= 0) return;
    int32_t xx = x / z + hvwidth;
    int32_t yy = hvheight - ((hvheight - (hvheight >> 3)) << 22) / z;
    if(xx > -10 && xx < viewwidth)
    {
        int stop = 10;
        int i = 0;
        if(xx < 0) i = -xx;
        if(xx > viewwidth - 11) stop = viewwidth - xx;
        for(; i < stop; i++)
            for(int j = 0; j < 10; j++)
                vbuf[(yy + j) * vbufPitch + xx + i] = moon[j * 10 + i];
    }*/
}

#endif

#ifdef USE_RAIN

void DrawRain(byte *vbuf, uint32_t vbufPitch)
{
    fixed dist;                                // distance to row projection
    fixed tex_step;                            // global step per one screen pixel
    fixed gu, gv, floorx, floory;              // global texture coordinates
    fixed px = (player->y + FixedMul(0x7900, viewsin)) >> 6;
    fixed pz = (player->x - FixedMul(0x7900, viewcos)) >> 6;
    int32_t ax, az, x, y, z, xx, yy, height, actheight;
    int shade;
    int hvheight = viewheight >> 1;
    int hvwidth = viewwidth >> 1;

    rainpos -= (tics * 900);
    for(int i = 0; i < MAXPOINTS; i++)
    {

        point3d_t *pt = &points[i];
        ax = pt->x + px;
        ax = 0x1fff - (ax & 0x3fff);
        az = pt->z + pz;
        az = 0x1fff - (az & 0x3fff);
        x = ax * viewcos + az * viewsin;
        y = -(heightnumerator << 7) + ((((pt->y << 6) + rainpos) & 0x0ffff) << 11);
        z = (az * viewcos - ax * viewsin) >> 8;
        if(z <= 0) continue;
        shade = z >> 17;
        if(shade > 13) continue;
        xx = x / z + hvwidth;
        if(xx < 0 || xx >= viewwidth) continue;
        actheight = y / z;
        yy = hvheight - actheight;
        height = (heightnumerator << 10) / z;
        if(actheight < 0) actheight = -actheight;
        if(actheight < (wallheight[xx] >> 3) && height < wallheight[xx]) continue;

#ifdef FIXLEAKS
            // Find the rain's tile coordinate
            // NOTE: This sometimes goes over the map edges.
            dist=((heightnumerator/((height>>3)+1))<<5);
            gu= viewx+FixedMul(dist, viewcos);
            gv=-viewy+FixedMul(dist, viewsin);
            floorx = (  gu>>TILESHIFT   )&63;
            floory = (-(gv>>TILESHIFT)-1)&63;
#endif

        if(xx >= 0 && xx < viewwidth && yy > 0 && yy < viewheight)
#ifdef FIXLEAKS
                {
                unsigned curtex = MAPSPOT(floorx, floory, 2)>>8;   // Is there a ceiling tile?
                if(!curtex)
#endif
        {
#ifdef TRANSPARENTRAIN
            vbuf[yy * vbufPitch + xx] = mergetable[shade][vbuf[yy * vbufPitch + xx]];
            vbuf[(yy - 1) * vbufPitch + xx] = mergetable[shade][vbuf[(yy - 1) * vbufPitch + xx]];
            if(yy > 2)
                vbuf[(yy - 2) * vbufPitch + xx] = mergetable[shade][vbuf[(yy - 2) * vbufPitch + xx]];
#else
            vbuf[yy * vbufPitch + xx] = shade+15;
            vbuf[(yy - 1) * vbufPitch + xx] = shade+16;
            if(yy > 2)
                vbuf[(yy - 2) * vbufPitch + xx] = shade+17;
#endif
        }
    }
}

#ifdef FIXLEAKS
}
#endif

#endif

#ifdef USE_SNOW

void DrawSnow(byte *vbuf, uint32_t vbufPitch)
{
    fixed dist;                                // distance to row projection
    fixed tex_step;                            // global step per one screen pixel
    fixed gu, gv, floorx, floory;              // global texture coordinates
    fixed px = (player->y + FixedMul(0x7900, viewsin)) >> 6;
    fixed pz = (player->x - FixedMul(0x7900, viewcos)) >> 6;
    int32_t ax, az, x, y, z, xx, yy, height, actheight;
    int shade;
    int hvheight = viewheight >> 1;
    int hvwidth = viewwidth >> 1;

    rainpos -= (tics * 256);
    for(int i = 0; i < MAXPOINTS; i++)
    {
        point3d_t *pt = &points[i];
        ax = pt->x + px;
        ax = 0x1fff - (ax & 0x3fff);
        az = pt->z + pz;
        az = 0x1fff - (az & 0x3fff);
        x = ax * viewcos + az * viewsin;
        y = -(heightnumerator << 7) + ((((pt->y << 6) + rainpos) & 0x0ffff) << 11);
        z = (az * viewcos - ax * viewsin) >> 8;
        if(z <= 0) continue;
        shade = z >> 17;
        if(shade > 13) continue;
        xx = x / z + hvwidth;
        if(xx < 0 || xx >= viewwidth) continue;
        actheight = y / z;
        yy = hvheight - actheight;
        height = (heightnumerator << 10) / z;
        if(actheight < 0) actheight = -actheight;
        if(actheight < (wallheight[xx] >> 3) && height < wallheight[xx]) continue;

#ifdef FIXLEAKS
            // Find the rain's tile coordinate
            // NOTE: This sometimes goes over the map edges.
            dist=((heightnumerator/((height>>3)+1))<<5);
            gu= viewx+FixedMul(dist, viewcos);
            gv=-viewy+FixedMul(dist, viewsin);
            floorx = (  gu>>TILESHIFT   )&63;
            floory = (-(gv>>TILESHIFT)-1)&63;
#endif

        if(xx > 0 && xx < viewwidth && yy > 0 && yy < viewheight)
#ifdef FIXLEAKS
                {
                unsigned curtex = MAPSPOT(floorx, floory, 2)>>8;   // Is there a ceiling tile?
                if(!curtex)
#endif
        {
            if(shade < 10)
            {
                vbuf[yy * vbufPitch + xx] = shade+87;
                vbuf[yy * vbufPitch + xx - 1] = shade+86;
                vbuf[(yy - 1) * vbufPitch + xx] = shade+86;
                vbuf[(yy - 1) * vbufPitch + xx - 1] = shade+85;
            }
            else
                vbuf[yy * vbufPitch + xx] = shade+85;
        }
    }
}

#ifdef FIXLEAKS
}
#endif

#endif 

