// WL_DRAW.C

#include "wl_def.h"
#pragma hdrstop

#include "wl_cloudsky.h"
#include "wl_atmos.h"
#include "wl_shade.h"

/*
=============================================================================

                               LOCAL CONSTANTS

=============================================================================
*/

// the door is the last picture before the sprites
#define DOORWALL        (PMSpriteStart-8)

#define ACTORSIZE       0x4000


/*
=============================================================================

                              GLOBAL VARIABLES

=============================================================================
*/

// BEGIN ZOOM BLOCK

// Global definitions
#ifndef   FOCALLENGTH
#define   FOCALLENGTH            0x5700L               // MUST match the FOCALLENGTH definition in WL_MAIN.C
#endif
#define   FOCALTHRESHOLD         FOCALLENGTH            // Greater lengths won't change the view position

// END ZOOM BLOCK

static byte *vbuf = NULL;
unsigned vbufPitch = 0;

int   messagetime=0;    

int32_t    lasttimecount;
int32_t    frameon;
boolean fpscounter;

int fps_frames=0, fps_time=0, fps=0;

int *wallheight;
int min_wallheight;

//
// math tables
//
short *pixelangle;
int32_t finetangent[FINEANGLES/4];
fixed sintable[ANGLES+ANGLES/4];
fixed *costable = sintable+(ANGLES/4);

//
// refresh variables
//
fixed   viewx,viewy;                    // the focal point
short   viewangle;
fixed   viewsin,viewcos;

void    TransformActor (objtype *ob);
void    BuildTables (void);
void    ClearScreen (void);
int     CalcRotate (objtype *ob);
void    DrawScaleds (void);
void    CalcTics (void);
void    ThreeDRefresh (void);
void    DrawSniperCross (void);


//
// wall optimization variables
//
int     lastside;               // true for vertical
int32_t    lastintercept;
int     lasttilehit;
int     lasttexture;

//
// ray tracing variables
//
short    focaltx,focalty,viewtx,viewty;
longword xpartialup,xpartialdown,ypartialup,ypartialdown;

short   midangle,angle;

word    tilehit;
int     pixx;
#ifdef ALULIGHTING
bool    outsidepost;
#endif


short   xtile,ytile;
short   xtilestep,ytilestep;
int32_t    xintercept,yintercept;
word    xstep,ystep;
word    xspot,yspot;
int     texdelta;
short   pw = 0;

word horizwall[MAXWALLTILES],vertwall[MAXWALLTILES];
int doorsets[7][5] = {
    { 0, -2, -10, -18, -26},
    { 6, -8, -16, -24, -30},
    { 6, -8, -16, -24, -30},
    { 6, -8, -16, -24, -30},
    { 6, -8, -16, -24, -30},
    { 6, -8, -16, -24, -30},
    { 4, -6, -14, -22, -30}
};

int doorsides[5] = {
    2, -4, -12, -20, -28
};
/*
============================================================================

                           3 - D  DEFINITIONS

============================================================================
*/

#define IsOutside(x, y, step, v) (((!v) ? areatable[x][y-step] : areatable[x-step][y]) & TL_OUTDOOR)

/*
========================
=
= TransformActor
=
= Takes paramaters:
=   gx,gy               : globalx/globaly of point
=
= globals:
=   viewx,viewy         : point of view
=   viewcos,viewsin     : sin/cos of viewangle
=   scale               : conversion from global value to screen value
=
= sets:
=   screenx,transx,transy,screenheight: projected edge location and size
=
========================
*/


//
// transform actor
//
void TransformActor (objtype *ob)
{
    fixed gx,gy,gxt,gyt,nx,ny;

//
// translate point to view centered coordinates
//
    gx = ob->x-viewx;
    gy = ob->y-viewy;

//
// calculate newx
//
    gxt = FixedMul(gx,viewcos);
    gyt = FixedMul(gy,viewsin);
    nx = gxt-gyt-ACTORSIZE;         // fudge the shape forward a bit, because
                                    // the midpoint could put parts of the shape
                                    // into an adjacent wall

//
// calculate newy
//
    gxt = FixedMul(gx,viewsin);
    gyt = FixedMul(gy,viewcos);
    ny = gyt+gxt;

//
// calculate perspective ratio
//
    ob->transx = nx;
    ob->transy = ny;

    if (nx<MINDIST)                 // too close, don't overflow the divide
    {
        ob->viewheight = 0;
        return;
    }

    ob->viewx = (word)(centerx + ny*scale/nx);

//
// calculate height (heightnumerator/(nx>>8))
//
    ob->viewheight = (word)(heightnumerator/(nx>>8));
}

//==========================================================================

/*
========================
=
= TransformTile
=
= Takes paramaters:
=   tx,ty               : tile the object is centered in
=
= globals:
=   viewx,viewy         : point of view
=   viewcos,viewsin     : sin/cos of viewangle
=   scale               : conversion from global value to screen value
=
= sets:
=   screenx,transx,transy,screenheight: projected edge location and size
=
= Returns true if the tile is withing getting distance
=
========================
*/

boolean TransformTile (int tx, int ty, short *dispx, short *dispheight)
{
    fixed gx,gy,gxt,gyt,nx,ny;

//
// translate point to view centered coordinates
//
    gx = ((int32_t)tx<<TILESHIFT)+0x8000-viewx;
    gy = ((int32_t)ty<<TILESHIFT)+0x8000-viewy;

//
// calculate newx
//
    gxt = FixedMul(gx,viewcos);
    gyt = FixedMul(gy,viewsin);
    nx = gxt-gyt-0x2000;            // 0x2000 is size of object

//
// calculate newy
//
    gxt = FixedMul(gx,viewsin);
    gyt = FixedMul(gy,viewcos);
    ny = gyt+gxt;


//
// calculate height / perspective ratio
//
    if (nx<MINDIST)                 // too close, don't overflow the divide
        *dispheight = 0;
    else
    {
        *dispx = (short)(centerx + ny*scale/nx);
        *dispheight = (short)(heightnumerator/(nx>>8));
    }

//
// see if it should be grabbed
//
    if (nx<TILEGLOBAL && ny>-TILEGLOBAL/2 && ny<TILEGLOBAL/2)
        return true;
    else
        return false;
}

//==========================================================================

/*
====================
=
= CalcHeight
=
= Calculates the height of xintercept,yintercept from viewx,viewy
=
====================
*/

int CalcHeight()
{
    fixed z = FixedMul(xintercept - viewx, viewcos)
        - FixedMul(yintercept - viewy, viewsin);
    if(z < MINDIST) z = MINDIST;
    int height = heightnumerator / (z >> 8);
    if(height < min_wallheight) min_wallheight = height;
    return height;
}

//==========================================================================

/*
===================
=
= ScalePost
=
===================
*/

byte *postsource;
int postx;
int postwidth;

void ScalePost()
{
    int ywcount, yoffs, yw, yd, yendoffs;
    byte col;

#ifdef USE_SHADING
#ifdef ALULIGHTING
    byte *curshades;
    if (!outsidepost)
        curshades = indoortable[GetShade(wallheight[postx], 1)];
    else
        curshades = shadetable[GetShade(wallheight[postx], 0)];
#else
    byte *curshades = shadetable[GetShade(wallheight[postx])];
#endif
#endif

    ywcount = yd = wallheight[postx] >> 3;
    if(yd <= 0) yd = 100;

    yoffs = (viewheight / 2 - ywcount) * vbufPitch;
    if(yoffs < 0) yoffs = 0;
    yoffs += postx;

    yendoffs = viewheight / 2 + ywcount - 1;
    yw=TEXTURESIZE-1;

    while(yendoffs >= viewheight)
    {
        ywcount -= TEXTURESIZE/2;
        while(ywcount <= 0)
        {
            ywcount += yd;
            yw--;
        }
        yendoffs--;
    }
    if(yw < 0) return;

#ifdef USE_SHADING
    col = curshades[postsource[yw]];
#else
    col = postsource[yw];
#endif
    yendoffs = yendoffs * vbufPitch + postx;
    while(yoffs <= yendoffs)
    {
        vbuf[yendoffs] = col;
        ywcount -= TEXTURESIZE/2;
        if(ywcount <= 0)
        {
            do
            {
                ywcount += yd;
                yw--;
            }
            while(ywcount <= 0);
            if(yw < 0) break;
#ifdef USE_SHADING
            col = curshades[postsource[yw]];
#else
            col = postsource[yw];
#endif
        }
        yendoffs -= vbufPitch;
    }
}

void GlobalScalePost(byte *vidbuf, unsigned pitch)
{
    vbuf = vidbuf;
    vbufPitch = pitch;
    ScalePost();
}

/*
====================
=
= HitVertWall
=
= tilehit bit 7 is 0, because it's not a door tile
= if bit 6 is 1 and the adjacent tile is a door tile, use door side pic
=
====================
*/

void HitVertWall (void)
{
    int wallpic;
    int texture;
    
    ytile = (short) (yintercept >> TILESHIFT);
    texture = ((yintercept+texdelta)>>TEXTUREFROMFIXEDSHIFT)&TEXTUREMASK;
    if (xtilestep == -1)
    {
        texture = TEXTUREMASK-texture;
        xintercept += TILEGLOBAL;
    }

    if(lastside==1 && lastintercept==xtile && lasttilehit==tilehit && !(lasttilehit & 0x40)
#ifdef ALULIGHTING
     && outsidepost == IsOutside(xtile, ytile, xtilestep, 1)
#endif
	)
    {
        if((pixx&3) && texture == lasttexture)
        {
            ScalePost();
            postx = pixx;
            wallheight[pixx] = wallheight[pixx-1];
            return;
        }
        ScalePost();
        wallheight[pixx] = CalcHeight();
        postsource+=texture-lasttexture;
        postwidth=1;
        postx=pixx;
        lasttexture=texture;
        return;
    }

    if(lastside!=-1) ScalePost();

    lastside=1;
    lastintercept=xtile;
    lasttilehit=tilehit;
    lasttexture=texture;
    wallheight[pixx] = CalcHeight();
    postx = pixx;
    postwidth = 1;
#ifdef ALULIGHTING
    outsidepost = IsOutside(xtile, ytile, xtilestep, 1);
#endif

    if (tilehit & 0x40)
    {                                                               // check for adjacent doors
        ytile = (short)(yintercept>>TILESHIFT);
        if ( tilemap[xtile-xtilestep][ytile]&0x80 )
            wallpic = DOORWALL + doorsides[doorset];
/*            switch (gamestate.mapon)
                        {
                         case 30:
                         case 40:
                         case 41:
                         case 46:
                         case 53:
                         wallpic = DOORWALL-28;
                         break;
                         case 32:
                         case 33:
                         case 34:
                         case 35:
                         case 36:
                         wallpic = DOORWALL-20;
                         break;
                         case 4:
                         case 22:
                         case 23:
                         case 31:
                         case 45:
                         wallpic = DOORWALL-4;
                         break;
                         case 5:
                         case 6:
                         case 7:
                         case 8:
                         case 9:
                         case 20:
                         case 21:
                         case 24:
                         case 26:
                         case 27:
                         case 37:
                         case 38:
                         case 39:
                         case 43:
                         wallpic = DOORWALL-12;
                         break;
                         default :
                         wallpic = DOORWALL+2;
                         break;
                           }*/
                        else
            wallpic = vertwall[tilehit & ~0x40];
    }
    else
        wallpic = vertwall[tilehit];

#ifdef ALULIGHTING
	if (wallpic > 218 && !outsidepost)
    {
        if (TILEISOUTSIDE(xtile-xtilestep, ytile + 1))
        {
            uint32_t ypoint = yintercept - (ytile << TILESHIFT);
            if (ypoint > 0x8000)
                outsidepost = true;
        }
        if (TILEISOUTSIDE(xtile-xtilestep, ytile - 1))
        {
            uint32_t ypoint = yintercept - (ytile << TILESHIFT);
            if (ypoint < 0x8000)
                outsidepost = true;
        }
    }
#endif

    postsource = PM_GetTexture(wallpic) + texture;
}


/*
====================
=
= HitHorizWall
=
= tilehit bit 7 is 0, because it's not a door tile
= if bit 6 is 1 and the adjacent tile is a door tile, use door side pic
=
====================
*/

void HitHorizWall (void)
{
    int wallpic;
    int texture;
    
    xtile = (short) (xintercept >> TILESHIFT);
    texture = ((xintercept+texdelta)>>TEXTUREFROMFIXEDSHIFT)&TEXTUREMASK;
    if (ytilestep == -1)
        yintercept += TILEGLOBAL;
    else
        texture = TEXTUREMASK-texture;

    if(lastside==0 && lastintercept==ytile && lasttilehit==tilehit && !(lasttilehit & 0x40)
#ifdef ALULIGHTING
     && outsidepost == IsOutside(xtile, ytile, ytilestep, 0)
#endif
	)
    {
        if((pixx&3) && texture == lasttexture)
        {
            ScalePost();
            postx=pixx;
            wallheight[pixx] = wallheight[pixx-1];
            return;
        }
        ScalePost();
        wallheight[pixx] = CalcHeight();
        postsource+=texture-lasttexture;
        postwidth=1;
        postx=pixx;
        lasttexture=texture;
        return;
    }

    if(lastside!=-1) ScalePost();

    lastside=0;
    lastintercept=ytile;
    lasttilehit=tilehit;
    lasttexture=texture;
    wallheight[pixx] = CalcHeight();
    postx = pixx;
    postwidth = 1;
#ifdef ALULIGHTING
	outsidepost = IsOutside(xtile, ytile, ytilestep, 0);
#endif

    if (tilehit & 0x40)
    {                                                               // check for adjacent doors
        xtile = (short)(xintercept>>TILESHIFT);
        if ( tilemap[xtile][ytile-ytilestep]&0x80)
            wallpic = DOORWALL + doorsides[doorset]+1;
/*            switch (gamestate.mapon)
                        {
                         case 30: // 4
                         case 40:
                         case 41:
                         case 46:
                         case 53:
                         wallpic = DOORWALL-27;
                         break;
                         case 32: // 3
                         case 33:
                         case 34:
                         case 35:
                         case 36:
                         wallpic = DOORWALL-19;
                         break;
                         case 4:  // 1
                         case 22:
                         case 23:
                         case 31:
                         case 45:
                         wallpic = DOORWALL-3;
                         break;
                         case 5: // 2
                         case 6:
                         case 7:
                         case 8:
                         case 9: 
                         case 20:
                         case 21:
                         case 24:
                         case 26:
                         case 27:
                         case 37:
                         case 38:
                         case 39:
                         case 43:
                         wallpic = DOORWALL-11;
                         break;
                         default : // 0
                         wallpic = DOORWALL+2;
                         break;
                           }*/
                        else
            wallpic = horizwall[tilehit & ~0x40];
    }
    else
        wallpic = horizwall[tilehit];

#ifdef ALULIGHTING
	if (wallpic > 218 && !outsidepost)
    {
        if (TILEISOUTSIDE(xtile + 1, ytile-ytilestep))
        {
            uint32_t xpoint = xintercept - (xtile << TILESHIFT);
            if (xpoint > 0x8000)
                outsidepost = true;
        }
        if (TILEISOUTSIDE(xtile - 1, ytile-ytilestep))
        {
            uint32_t xpoint = xintercept - (xtile << TILESHIFT);
            if (xpoint < 0x8000)
                outsidepost = true;
        }
    }
#endif

    postsource = PM_GetTexture(wallpic) + texture;
}

//==========================================================================

/*
====================
=
= HitHorizDoor
=
====================
*/

void HitHorizDoor (void)
{
    int doorpage;
    int doornum;
    int texture;
    
    doornum = tilehit&0x7f;
    texture = ((xintercept-doorposition[doornum])>>TEXTUREFROMFIXEDSHIFT)&TEXTUREMASK;

    if(lasttilehit==tilehit
#ifdef ALULIGHTING
     && outsidepost == IsOutside(xtile, ytile, ytilestep, 0)
#endif
	)
    {
        if((pixx&3) && texture == lasttexture)
        {
            ScalePost();
            postx=pixx;
            wallheight[pixx] = wallheight[pixx-1];
            return;
        }
        ScalePost();
        wallheight[pixx] = CalcHeight();
        postsource+=texture-lasttexture;
        postwidth=1;
        postx=pixx;
        lasttexture=texture;
        return;
    }

    if(lastside!=-1) ScalePost();

    lastside=2;
    lasttilehit=tilehit;
    lasttexture=texture;
    wallheight[pixx] = CalcHeight();
    postx = pixx;
    postwidth = 1;
#ifdef ALULIGHTING
    outsidepost = IsOutside(xtile, ytile, ytilestep, 0);
#endif
    doorpage = DOORWALL + doorsets[doorobjlist[doornum].lock][doorset];

/*    switch(doorobjlist[doornum].lock)
    {
        case dr_normal:
            if (gamestate.mapon == 30 || gamestate.mapon == 40 || gamestate.mapon == 41 || gamestate.mapon == 46 || gamestate.mapon == 53)
                        doorpage = DOORWALL-26;
                else
            if (gamestate.mapon == 32 || gamestate.mapon == 33 || gamestate.mapon == 34 || gamestate.mapon == 35 || gamestate.mapon == 36)
                        doorpage = DOORWALL-18;
                else
             if (gamestate.mapon == 5 || gamestate.mapon == 6 || gamestate.mapon == 7 || gamestate.mapon == 8 || gamestate.mapon == 9 || gamestate.mapon == 20 || gamestate.mapon == 21 || gamestate.mapon == 26 || gamestate.mapon == 27 || gamestate.mapon == 24 || gamestate.mapon == 37 || gamestate.mapon == 38 || gamestate.mapon == 39 || gamestate.mapon == 43)
                        doorpage = DOORWALL-10;
                else
             if (gamestate.mapon == 4 || gamestate.mapon == 22 || gamestate.mapon == 23 || gamestate.mapon == 31 || gamestate.mapon == 45)
                        doorpage = DOORWALL-2;
                else
            doorpage = DOORWALL;
            break;
        case dr_lock1:
        case dr_lock2:
        case dr_lock3:
        case dr_lock4:
        case dr_lock5:
            if (gamestate.mapon == 30 || gamestate.mapon == 40 || gamestate.mapon == 41 || gamestate.mapon == 46 || gamestate.mapon == 53)
                        doorpage = DOORWALL-30;
                else
            if (gamestate.mapon == 32 || gamestate.mapon == 33 || gamestate.mapon == 34 || gamestate.mapon == 35 || gamestate.mapon == 36)
                        doorpage = DOORWALL-24;
                else
             if (gamestate.mapon == 5 || gamestate.mapon == 6 || gamestate.mapon == 7 || gamestate.mapon == 8 || gamestate.mapon == 9 || gamestate.mapon == 20 || gamestate.mapon == 21 || gamestate.mapon == 26 || gamestate.mapon == 27 || gamestate.mapon == 24 || gamestate.mapon == 37 || gamestate.mapon == 38 || gamestate.mapon == 39 || gamestate.mapon == 43)
                        doorpage = DOORWALL-16;
                else
             if (gamestate.mapon == 4 || gamestate.mapon == 22 || gamestate.mapon == 23 || gamestate.mapon == 31 || gamestate.mapon == 45)
                        doorpage = DOORWALL-8;
                else
            doorpage = DOORWALL+6;
            break;
        case dr_elevator:
            if (gamestate.mapon == 30 || gamestate.mapon == 40 || gamestate.mapon == 41 || gamestate.mapon == 46 || gamestate.mapon == 53)
                        doorpage = DOORWALL-30;
                else
            if (gamestate.mapon == 32 || gamestate.mapon == 33 || gamestate.mapon == 34 || gamestate.mapon == 35 || gamestate.mapon == 36)
                        doorpage = DOORWALL-22;
                else
             if (gamestate.mapon == 5 || gamestate.mapon == 6 || gamestate.mapon == 7 || gamestate.mapon == 8 || gamestate.mapon == 9 || gamestate.mapon == 20 || gamestate.mapon == 21 || gamestate.mapon == 26 || gamestate.mapon == 27 || gamestate.mapon == 24 || gamestate.mapon == 37 || gamestate.mapon == 38 || gamestate.mapon == 39 || gamestate.mapon == 43)
                        doorpage = DOORWALL-14;
                else
             if (gamestate.mapon == 4 || gamestate.mapon == 22 || gamestate.mapon == 23 || gamestate.mapon == 31 || gamestate.mapon == 45)
                        doorpage = DOORWALL-6;
                else
            doorpage = DOORWALL+4;
            break;
    }*/

    postsource = PM_GetTexture(doorpage) + texture;
}

//==========================================================================

/*
====================
=
= HitVertDoor
=
====================
*/

void HitVertDoor (void)
{
    int doorpage;
    int doornum;
    int texture;
    
    doornum = tilehit&0x7f;
    texture = ((yintercept-doorposition[doornum])>>TEXTUREFROMFIXEDSHIFT)&TEXTUREMASK;

    if(lasttilehit==tilehit
#ifdef ALULIGHTING
     && outsidepost == IsOutside(xtile, ytile, xtilestep, 1)
#endif
	)
    {
        if((pixx&3) && texture == lasttexture)
        {
            ScalePost();
            postx=pixx;
            wallheight[pixx] = wallheight[pixx-1];
            return;
        }
        ScalePost();
        wallheight[pixx] = CalcHeight();
        postsource+=texture-lasttexture;
        postwidth=1;
        postx=pixx;
        lasttexture=texture;
        return;
    }

    if(lastside!=-1) ScalePost();

    lastside=2;
    lasttilehit=tilehit;
    lasttexture=texture;
    wallheight[pixx] = CalcHeight();
    postx = pixx;
    postwidth = 1;
#ifdef ALULIGHTING
    outsidepost = IsOutside(xtile, ytile, xtilestep, 1);
#endif
    doorpage = DOORWALL + doorsets[doorobjlist[doornum].lock][doorset]+1;

/*    switch(doorobjlist[doornum].lock)
    {
         case dr_normal: 
            if (gamestate.mapon == 30 || gamestate.mapon == 40 || gamestate.mapon == 41 || gamestate.mapon == 46 || gamestate.mapon == 53)
                        doorpage = DOORWALL-25;
                else       
            if (gamestate.mapon == 32 || gamestate.mapon == 33 || gamestate.mapon == 34 || gamestate.mapon == 35 || gamestate.mapon == 36)
                        doorpage = DOORWALL-17;
                else
             if (gamestate.mapon == 5 || gamestate.mapon == 6 || gamestate.mapon == 7 || gamestate.mapon == 8 || gamestate.mapon == 9 || gamestate.mapon == 20 || gamestate.mapon == 21 || gamestate.mapon == 26 || gamestate.mapon == 27 || gamestate.mapon == 24 || gamestate.mapon == 37 || gamestate.mapon == 38 || gamestate.mapon == 39 || gamestate.mapon == 43)
                        doorpage = DOORWALL-9;
                else
             if (gamestate.mapon == 4 || gamestate.mapon == 22 || gamestate.mapon == 23 || gamestate.mapon == 31 || gamestate.mapon == 45)
                        doorpage = DOORWALL-1;
                else
            doorpage = DOORWALL+1;
            break;
        case dr_lock1:
        case dr_lock2:
        case dr_lock3:
        case dr_lock4:
        case dr_lock5:
            if (gamestate.mapon == 30 || gamestate.mapon == 40 || gamestate.mapon == 41 || gamestate.mapon == 46 || gamestate.mapon == 53)
                        doorpage = DOORWALL-31;
                else
            if (gamestate.mapon == 32 || gamestate.mapon == 33 || gamestate.mapon == 34 || gamestate.mapon == 35 || gamestate.mapon == 36)
                        doorpage = DOORWALL-23;
                else
             if (gamestate.mapon == 5 || gamestate.mapon == 6 || gamestate.mapon == 7 || gamestate.mapon == 8 || gamestate.mapon == 9 || gamestate.mapon == 20 || gamestate.mapon == 21 || gamestate.mapon == 26 || gamestate.mapon == 27 || gamestate.mapon == 24 || gamestate.mapon == 37 || gamestate.mapon == 38 || gamestate.mapon == 39 || gamestate.mapon == 43)
                        doorpage = DOORWALL-15;
                else
             if (gamestate.mapon == 4 || gamestate.mapon == 22 || gamestate.mapon == 23 || gamestate.mapon == 31 || gamestate.mapon == 45)
                        doorpage = DOORWALL-7;
                else
            doorpage = DOORWALL+7;
            break;
        case dr_elevator:
            if (gamestate.mapon == 30 || gamestate.mapon == 40 || gamestate.mapon == 41 || gamestate.mapon == 46 || gamestate.mapon == 53)
                        doorpage = DOORWALL-29;
                else
            if (gamestate.mapon == 32 || gamestate.mapon == 33 || gamestate.mapon == 34 || gamestate.mapon == 35 || gamestate.mapon == 36)
                        doorpage = DOORWALL-21;
                else
             if (gamestate.mapon == 5 || gamestate.mapon == 6 || gamestate.mapon == 7 || gamestate.mapon == 8 || gamestate.mapon == 9 || gamestate.mapon == 20 || gamestate.mapon == 21 || gamestate.mapon == 26 || gamestate.mapon == 27 || gamestate.mapon == 24 || gamestate.mapon == 37 || gamestate.mapon == 38 || gamestate.mapon == 39 || gamestate.mapon == 43)
                        doorpage = DOORWALL-13;
                else
             if (gamestate.mapon == 4 || gamestate.mapon == 22 || gamestate.mapon == 23 || gamestate.mapon == 31 || gamestate.mapon == 45)
                        doorpage = DOORWALL-5;
                else
            doorpage = DOORWALL+5;
            break;
    }*/

    postsource = PM_GetTexture(doorpage) + texture;
}

//==========================================================================


/*
====================
=
= HitHorizPWall
=
= A pushable wall in action has been hit
=
====================
*/

void HitHorizPWall (void)
{
    int wallpic;
    int texture,offset;

    texture = (xintercept>>TEXTUREFROMFIXEDSHIFT)&TEXTUREMASK;
    offset = pwallpos<<10;
    if (ytilestep == -1)
        yintercept += TILEGLOBAL-offset;
    else
    {
        texture = TEXTUREMASK-texture;
        yintercept += offset;
    }

    if(lasttilehit==tilehit && lastside==0
#ifdef ALULIGHTING
     && outsidepost == IsOutside(xtile, ytile, ytilestep, 0)
#endif
	)
    {
        if((pixx&3) && texture == lasttexture)
        {
            ScalePost();
            postx=pixx;
            wallheight[pixx] = wallheight[pixx-1];
            return;
        }
        ScalePost();
        wallheight[pixx] = CalcHeight();
        postsource+=texture-lasttexture;
        postwidth=1;
        postx=pixx;
        lasttexture=texture;
        return;
    }

    if(lastside!=-1) ScalePost();

    lastside=0;
    lasttilehit=tilehit;
    lasttexture=texture;
    wallheight[pixx] = CalcHeight();
    postx = pixx;
    postwidth = 1;
#ifdef ALULIGHTING
	outsidepost = IsOutside(xtile, ytile, ytilestep, 0);
#endif

    wallpic = horizwall[pwalltile&63];

    postsource = PM_GetTexture(wallpic) + texture;
}

/*
====================
=
= HitVertPWall
=
= A pushable wall in action has been hit
=
====================
*/

void HitVertPWall (void)
{
    int wallpic;
    int texture,offset;

    texture = (yintercept>>TEXTUREFROMFIXEDSHIFT)&TEXTUREMASK;
    offset = pwallpos<<10;
    if (xtilestep == -1)
    {
        xintercept += TILEGLOBAL-offset;
        texture = TEXTUREMASK-texture;
    }
    else
        xintercept += offset;

    if(lasttilehit==tilehit && lastside==1
#ifdef ALULIGHTING
     && outsidepost == IsOutside(xtile, ytile, xtilestep, 1)
#endif
	)
    {
        if((pixx&3) && texture == lasttexture)
        {
            ScalePost();
            postx=pixx;
            wallheight[pixx] = wallheight[pixx-1];
            return;
        }
        ScalePost();
        wallheight[pixx] = CalcHeight();
        postsource+=texture-lasttexture;
        postwidth=1;
        postx=pixx;
        lasttexture=texture;
        return;
    }

    if(lastside!=-1) ScalePost();

    lastside=1;
    lasttilehit=tilehit;
    lasttexture=texture;
    wallheight[pixx] = CalcHeight();
    postx = pixx;
    postwidth = 1;
#ifdef ALULIGHTING
	outsidepost = IsOutside(xtile, ytile, xtilestep, 1);
#endif

    wallpic = vertwall[pwalltile&63];

    postsource = PM_GetTexture(wallpic) + texture;
}

#define HitHorizBorder HitHorizWall
#define HitVertBorder HitVertWall

//==========================================================================

byte vgaCeiling[]=
{
#ifndef SPEAR
 0x1d,0x1d,0x1d,0x1d,0x1d,0x1d,0x1d,0x1d,0x1d,0xbf,
 0x4e,0x4e,0x4e,0x1d,0x8d,0x4e,0x1d,0x2d,0x1d,0x8d,
 0x1d,0x1d,0x1d,0x1d,0x1d,0x2d,0xdd,0x1d,0x1d,0x98,

 0x1d,0x9d,0x2d,0xdd,0xdd,0x9d,0x2d,0x4d,0x1d,0xdd,
 0x7d,0x1d,0x2d,0x2d,0xdd,0xd7,0x1d,0x1d,0x1d,0x2d,
 0x1d,0x1d,0x1d,0x1d,0xdd,0xdd,0x7d,0xdd,0xdd,0xdd
#else
 0x6f,0x4f,0x1d,0xde,0xdf,0x2e,0x7f,0x9e,0xae,0x7f,
 0x1d,0xde,0xdf,0xde,0xdf,0xde,0xe1,0xdc,0x2e,0x1d,0xdc
#endif
};

/*
=====================
=
= VGAClearScreen
=
=====================
*/

void VGAClearScreen (void)
{
    byte ceiling=vgaCeiling[gamestate.episode*10+mapon];

    int y;
    byte *ptr = vbuf;
#ifdef USE_SHADING
#ifdef ALULIGHTING
    for(y = 0; y < viewheight / 2; y++, ptr += vbufPitch)
        memset(ptr, shadetable[GetShade((viewheight / 2 - y) << 3, 0)][ceiling], viewwidth);
    for(; y < viewheight; y++, ptr += vbufPitch)
        memset(ptr, shadetable[GetShade((y - viewheight / 2) << 3, 0)][0x19], viewwidth);
#else
    for(y = 0; y < viewheight / 2; y++, ptr += vbufPitch)
        memset(ptr, shadetable[GetShade((viewheight / 2 - y) << 3)][ceiling], viewwidth);
    for(; y < viewheight; y++, ptr += vbufPitch)
        memset(ptr, shadetable[GetShade((y - viewheight / 2) << 3)][0x19], viewwidth);
#endif
#else
    for(y = 0; y < viewheight / 2; y++, ptr += vbufPitch)
        memset(ptr, ceiling, viewwidth);
    for(; y < viewheight; y++, ptr += vbufPitch)
        memset(ptr, 0x19, viewwidth);
#endif
}

//==========================================================================

/*
=====================
=
= CalcRotate
=
=====================
*/

int CalcRotate (objtype *ob)
{
    int angle, viewangle;

    // this isn't exactly correct, as it should vary by a trig value,
    // but it is close enough with only eight rotations

    viewangle = player->angle + (centerx - ob->viewx)/8;

    if (ob->obclass == rocketobj || ob->obclass == drocketobj || ob->obclass == hrocketobj || ob->obclass == gasballobj)
        angle = (viewangle-180) - ob->angle;
    else
        angle = (viewangle-180) - dirangle[ob->dir];

    angle+=ANGLES/16;
    while (angle>=ANGLES)
        angle-=ANGLES;
    while (angle<0)
        angle+=ANGLES;

    if (ob->state->rotate == 2)             // 2 rotation pain frame
        return 0;               // pain with shooting frame bugfix

    return angle/(ANGLES/8);
}

void ScaleShape (int xcenter, int shapenum, unsigned height, uint32_t flags)
{
    t_compshape *shape;
    unsigned scale,pixheight;
    unsigned starty,endy;
    word *cmdptr;
    byte *cline;
    byte *line;
    byte *vmem;
    int actx,i,upperedge;
    short newstart;
    int scrstarty,screndy,lpix,rpix,pixcnt,ycnt;
    unsigned j;
    byte col;

#ifdef USE_SHADING
    byte *curshades;
#ifdef ALULIGHTING
    if (flags & FL_INDOOR)
    {
        if (flags & FL_FULLBRIGHT)
            curshades = indoortable[0];
        else
            curshades = indoortable[GetShade(height, 1)];
    }
    else
    {
        if (flags & FL_FULLBRIGHT)
            curshades = shadetable[0];
        else
            curshades = shadetable[GetShade(height, 0)];
    }
#else
    curshades = shadetable[GetShade(height)];
#endif
#endif

    shape = (t_compshape *) PM_GetSprite(shapenum);

    scale=height>>3;                 // low three bits are fractional
    if(!scale) return;   // too close or far away

    pixheight=scale*SPRITESCALEFACTOR;
    actx=xcenter-scale;
    upperedge=viewheight/2-scale;

    cmdptr=(word *) shape->dataofs;

    for(i=shape->leftpix,pixcnt=i*pixheight,rpix=(pixcnt>>6)+actx;i<=shape->rightpix;i++,cmdptr++)
    {
        lpix=rpix;
        if(lpix>=viewwidth) break;
        pixcnt+=pixheight;
        rpix=(pixcnt>>6)+actx;
        if(lpix!=rpix && rpix>0)
        {
            if(lpix<0) lpix=0;
            if(rpix>viewwidth) rpix=viewwidth,i=shape->rightpix+1;
            cline=(byte *)shape + *cmdptr;
            while(lpix<rpix)
            {
                if(wallheight[lpix]<=(int)height)
                {
                    line=cline;
                    while((endy = READWORD(line)) != 0)
                    {
                        endy >>= 1;
                        newstart = READWORD(line);
                        starty = READWORD(line) >> 1;
                        j=starty;
                        ycnt=j*pixheight;
                        screndy=(ycnt>>6)+upperedge;
                        if(screndy<0) vmem=vbuf+lpix;
                        else vmem=vbuf+screndy*vbufPitch+lpix;
                        for(;j<endy;j++)
                        {
                            scrstarty=screndy;
                            ycnt+=pixheight;
                            screndy=(ycnt>>6)+upperedge;
                            if(scrstarty!=screndy && screndy>0)
                            {
#ifdef USE_SHADING
                                col=curshades[((byte *)shape)[newstart+j]];
#else
                                col=((byte *)shape)[newstart+j];
#endif
                                if(scrstarty<0) scrstarty=0;
                                if(screndy>viewheight) screndy=viewheight,j=endy;

                                while(scrstarty<screndy)
                                {
                                    *vmem=col;
                                    vmem+=vbufPitch;
                                    scrstarty++;
                                }
                            }
                        }
                    }
                }
                lpix++;
            }
        }
    }
}

void SimpleScaleShape (int xcenter, int shapenum, unsigned height, int spc) 
{
    t_compshape   *shape;
    unsigned scale,pixheight;
    unsigned starty,endy;
    word *cmdptr;
    byte *cline;
    byte *line;
    int actx,i,upperedge;
    short newstart;
    int scrstarty,screndy,lpix,rpix,pixcnt,ycnt;
    unsigned j;
    byte col;
    byte *vmem;

    shape = (t_compshape *) PM_GetSprite(shapenum);

    scale=height>>1;
    pixheight=scale/2;
    actx=xcenter-scale/6;
     if(spc==1)
       upperedge=viewheight-scale+(gamestate.bobber*2);
    else
       upperedge=viewheight-scale; 

    cmdptr=shape->dataofs;

    for(i=shape->leftpix,pixcnt=i*pixheight,rpix=(pixcnt>>6)+actx;i<=shape->rightpix;i++,cmdptr++)
    {
        lpix=rpix;
        if(lpix>=viewwidth) break;
        pixcnt+=pixheight;
        rpix=(pixcnt>>6)+actx;
        if(lpix!=rpix && rpix>0)
        {
            if(lpix<0) lpix=0;
            if(rpix>viewwidth) rpix=viewwidth,i=shape->rightpix+1;
            cline = (byte *)shape + *cmdptr;
            while(lpix<rpix)
            {
                line=cline;
                while((endy = READWORD(line)) != 0)
                {
                    endy >>= 1;
                    newstart = READWORD(line);
                    starty = READWORD(line) >> 1;
                    j=starty;
                    ycnt=j*pixheight;
                    screndy=(ycnt>>6)+upperedge;
                    if(screndy<0) vmem=vbuf+lpix;
                    else vmem=vbuf+screndy*vbufPitch+lpix;
                    for(;j<endy;j++)
                    {
                        scrstarty=screndy;
                        ycnt+=pixheight;
                        screndy=(ycnt>>6)+upperedge;
                        if(scrstarty!=screndy && screndy>0)
                        {
                            col=((byte *)shape)[newstart+j];
                            if(scrstarty<0) scrstarty=0;
                            if(screndy>viewheight) screndy=viewheight,j=endy;

                            while(scrstarty<screndy)
                            {
                                *vmem=col;
                                vmem+=vbufPitch;
                                scrstarty++;
                            }
                        }
                    }
                }
                lpix++;
            }
        }
    }
}

/*
=====================
=
= DrawScaleds
=
= Draws all objects that are visable
=
=====================
*/

#define MAXVISABLE 250

typedef struct
{
    short      viewx,
               viewheight,
               shapenum;
    short      flags;          // this must be changed to uint32_t, when you
                               // you need more than 16-flags for drawing
                               
#ifdef USE_DIR3DSPR
    statobj_t *transsprite;
#endif
} visobj_t;

visobj_t vislist[MAXVISABLE];
visobj_t *visptr,*visstep,*farthest;

void DrawScaleds (void)
{
    int      i,least,numvisable,height;
    byte     *tilespot,*visspot;
    unsigned spotloc;

    statobj_t *statptr;
    objtype   *obj;

    visptr = &vislist[0];
    
    // Animated Objects code pt. 1
    frameon+=tics;
    frameon%=2520;
// End Animated Objects code pt. 2 

//
// place static objects
//
    for (statptr = &statobjlist[0] ; statptr !=laststatobj ; statptr++)
    {
        statptr->flags &= ~FL_INDOOR;
        if ((visptr->shapenum = statptr->shapenum) == -1)
            continue;                                               // object has been deleted

        if (!*statptr->visspot)
            continue;                                               // not visable
            
            // Animated Objects code pt.2
      switch (statptr->shapenum)
      {
      case SPR_STAT_78:                                // First animated object
           visptr->shapenum+=int((frameon/12)%4);
           break;
      case SPR_STAT_82:                                // Second animated object
           visptr->shapenum+=int((frameon/12)%4);
           break;
    case SPR_STAT_92:                                // Second animated object
           visptr->shapenum+=int((frameon/7)%4);
           break;
                }
// End Animated Objects code pt. 2 

        if (TransformTile (statptr->tilex,statptr->tiley,
            &visptr->viewx,&visptr->viewheight) && statptr->flags & FL_BONUS)
        {
            GetBonus (statptr);
            if(statptr->shapenum == -1)
                continue;                                           // object has been taken
        }

        if (!visptr->viewheight)
            continue; 
            
#ifdef ALULIGHTING
        if (MAPSPOT(statptr->tilex, statptr->tiley, 2) >> 8 != 0 && !(TILEISOUTSIDE(statptr->tilex, statptr->tiley)))
            statptr->flags |= FL_INDOOR;
#endif

#ifdef USE_DIR3DSPR
        if(statptr->flags & FL_DIR_MASK)
            visptr->transsprite=statptr;
        else
            visptr->transsprite=NULL;
#endif

        if (visptr < &vislist[MAXVISABLE-1])    // don't let it overflow
        {
            visptr->flags = (short) statptr->flags;
            visptr++;
        }
    }

//
// place active objects
//
    for (obj = player->next;obj;obj=obj->next)
    {
        obj->flags &= ~FL_INDOOR;
        if ((visptr->shapenum = obj->state->shapenum)==0)
            continue;                                               // no shape

        spotloc = (obj->tilex<<mapshift)+obj->tiley;   // optimize: keep in struct?
        visspot = &spotvis[0][0]+spotloc;
        tilespot = &tilemap[0][0]+spotloc;

        //
        // could be in any of the nine surrounding tiles
        //
        if (*visspot
            || ( *(visspot-1) && !*(tilespot-1) )
            || ( *(visspot+1) && !*(tilespot+1) )
            || ( *(visspot-65) && !*(tilespot-65) )
            || ( *(visspot-64) && !*(tilespot-64) )
            || ( *(visspot-63) && !*(tilespot-63) )
            || ( *(visspot+65) && !*(tilespot+65) )
            || ( *(visspot+64) && !*(tilespot+64) )
            || ( *(visspot+63) && !*(tilespot+63) ) )
        {
            obj->active = ac_yes;
            TransformActor (obj);
            if (!obj->viewheight)
                continue;                                               // too close or far away

            visptr->viewx = obj->viewx;
            visptr->viewheight = obj->viewheight;
            if (visptr->shapenum == -1)
                visptr->shapenum = obj->temp1;  // special shape

            if (obj->state->rotate)
                visptr->shapenum += CalcRotate (obj);
                
#ifdef ALULIGHTING
            if (MAPSPOT(obj->tilex, obj->tiley, 2) >> 8 != 0 && !(TILEISOUTSIDE(obj->tilex, obj->tiley)))
                obj->flags |= FL_INDOOR;
#endif

            if (visptr->shapenum >= SPR_BURN_1 && visptr->shapenum <= SPR_BURN_20)
                obj->flags |= FL_FULLBRIGHT;
            else
            {
                switch (visptr->shapenum)
                {
                    case SPR_VOFC_SHOOT3:
                    case SPR_SLV_SHOOT3:
                    case SPR_AFU_SHOOT3:
                    case SPR_BFU_SHOOT3:
                    case SPR_SKS_SHOOT3:
                    case SPR_SOU_SHOOT4:
                    case SPR_STH_SHOOT3:
                    case SPR_RKT_SHOOT3:
                    case SPR_SNI_SHOOT2:
                    case SPR_CUL_SHOOT2:
                    case SPR_SHT_SHOOT2:
                    case SPR_SPC_SHOOT3:
                    case SPR_GRD_SHOOT3:
                    case SPR_VET_SHOOT2:
                    case SPR_OFC_SHOOT3:
                    case SPR_SS_SHOOT3:
                    case SPR_BOSS_SHOOT2:
                    case SPR_BOSS_SHOOT3:
                    case SPR_GRETEL_SHOOT2:
                    case SPR_GRETEL_SHOOT3:
                    case SPR_SCHABB_SHOOT1:
                    case SPR_SCHABB_SHOOT2:
                    case SPR_FAT_SHOOT3:
                    case SPR_FAT_SHOOT4:
                    case SPR_FAKE_SHOOT:
                    case SPR_FAKE_SHOOT1:
                    case SPR_FAKE_SHOOT2:
                    case SPR_MECHA_SHOOT2:
                    case SPR_MECHA_SHOOT3:
                    case SPR_HITLER_SHOOT2:
                    case SPR_HITLER_SHOOT3:
                        obj->flags |= FL_FULLBRIGHT;
                        break;
                    default:
                        obj->flags &= ~FL_FULLBRIGHT;
                        break;
                }
            }    

            if (visptr < &vislist[MAXVISABLE-1])    // don't let it overflow
            {
                visptr->flags = (short) obj->flags;
#ifdef USE_DIR3DSPR
                visptr->transsprite = NULL;
#endif
                visptr++;
            }
            obj->flags |= FL_VISABLE;
        }
        else
            obj->flags &= ~FL_VISABLE;
    }

//
// draw from back to front
//
    numvisable = (int) (visptr-&vislist[0]);

    if (!numvisable)
        return;                                                                 // no visable objects

    for (i = 0; i<numvisable; i++)
    {
        least = 32000;
        for (visstep=&vislist[0] ; visstep<visptr ; visstep++)
        {
            height = visstep->viewheight;
            if (height < least)
            {
                least = height;
                farthest = visstep;
            }
        }
        //
        // draw farthest
        //
#ifdef USE_DIR3DSPR
        if(farthest->transsprite)
            Scale3DShape(vbuf, vbufPitch, farthest->transsprite);
        else
#endif
            ScaleShape(farthest->viewx, farthest->shapenum, farthest->viewheight, farthest->flags);

        farthest->viewheight = 32000;
    }
}

//==========================================================================

/*
==============
=
= DrawPlayerWeapon
=
= Draw the player's hands
=
==============
*/

int weaponscale[NUMWEAPONS] = {SPR_KNIFEREADY, SPR_PISTOLREADY, SPR_BERETTAREADY, SPR_MP5READY, SPR_TYPE79READY,
    SPR_MACHINEGUNREADY, SPR_LR3000READY, SPR_CHAINREADY, SPR_DSHOTREADY, SPR_SHOTREADY, SPR_L46READY, SPR_M14READY, SPR_BIZONREADY, SPR_AK47READY, SPR_DEAGLEREADY};
    
int   reloadscale[NUMWEAPONS] = {SPR_KNIFEREADY,SPR_PISTOL_RLOAD1,SPR_BERETTA_RLOAD1,SPR_MP5_RLOAD1,SPR_TYPE79_RLOAD1,
   SPR_MACHINEGUN_RLOAD1,SPR_LR3000_RLOAD1,SPR_CHAINGUN_RLOAD1,SPR_DSHOTGUN_RLOAD1,SPR_SHOTGUN_RLOAD1,SPR_L46_RLOAD1,SPR_M14_RLOAD1,SPR_BIZON_RLOAD1,SPR_AK47_RLOAD1,SPR_DEAGLE_RLOAD1}; 

void DrawPlayerWeapon (void)
{
    int shapenum;

#ifndef SPEAR
    if (gamestate.victoryflag)
    {
#ifndef APOGEE_1_0
        if (player->state == &s_deathcam && (GetTimeCount()&32) )
            SimpleScaleShape(viewwidth/2,SPR_DEATHCAM,viewheight+1, 0);
#endif
        return;
    }
#endif

   if (gamestate.curzoom && !gamestate.reloading)
	DrawSniperCross();
   else if (gamestate.weapon != -1)
   {
      if (gamestate.reloading)
            shapenum = reloadscale[gamestate.weapon]+gamestate.weaponframe;
        else
            shapenum = weaponscale[gamestate.weapon]+gamestate.weaponframe; 
#ifdef WEAPONBOBBLE
      SimpleScaleShape((viewwidth/2)-10+gamestate.bobber2,shapenum,viewheight+1,1);
    //   SimpleScaleShape((viewwidth/2)-10+gamestate.bobber2,shapenum,viewheight+1+(gamestate.bobber*2),0); 
#else
      SimpleScaleShape(viewwidth/2,shapenum,viewheight+1,0);
#endif 
    }

    if (demorecord || demoplayback)
        SimpleScaleShape(viewwidth/2,SPR_DEMO,viewheight+1,0);
}


//==========================================================================


/*
=====================
=
= CalcTics
=
=====================
*/

void CalcTics (void)
{
//
// calculate tics since last refresh for adaptive timing
//
    if (lasttimecount > (int32_t) GetTimeCount())
        lasttimecount = GetTimeCount();    // if the game was paused a LONG time

    uint32_t curtime = SDL_GetTicks();
    tics = (curtime * 7) / 100 - lasttimecount;
    if(!tics)
    {
        // wait until end of current tic
        SDL_Delay(((lasttimecount + 1) * 100) / 7 - curtime);
        tics = 1;
    }

    lasttimecount += tics;

    if (tics>MAXTICS)
        tics = MAXTICS;
}


//==========================================================================

#ifdef ALUMULTIPUSH
short GetCurPush(int x, int y)
{
	for (int i = 0; i < MAXPWALLS; i++)
    	if (pwall[i].x == x && pwall[i].y == y)
            return i;
}
#endif

void AsmRefresh()
{
    int32_t xstep,ystep;
    longword xpartial,ypartial;
    boolean playerInPushwallBackTile = tilemap[focaltx][focalty] == 64;

    for(pixx=0;pixx<viewwidth;pixx++)
    {
        short angl=midangle+pixelangle[pixx];
        if(angl<0) angl+=FINEANGLES;
        if(angl>=3600) angl-=FINEANGLES;
        if(angl<900)
        {
            xtilestep=1;
            ytilestep=-1;
            xstep=finetangent[900-1-angl];
            ystep=-finetangent[angl];
            xpartial=xpartialup;
            ypartial=ypartialdown;
        }
        else if(angl<1800)
        {
            xtilestep=-1;
            ytilestep=-1;
            xstep=-finetangent[angl-900];
            ystep=-finetangent[1800-1-angl];
            xpartial=xpartialdown;
            ypartial=ypartialdown;
        }
        else if(angl<2700)
        {
            xtilestep=-1;
            ytilestep=1;
            xstep=-finetangent[2700-1-angl];
            ystep=finetangent[angl-1800];
            xpartial=xpartialdown;
            ypartial=ypartialup;
        }
        else if(angl<3600)
        {
            xtilestep=1;
            ytilestep=1;
            xstep=finetangent[angl-2700];
            ystep=finetangent[3600-1-angl];
            xpartial=xpartialup;
            ypartial=ypartialup;
        }
        yintercept=FixedMul(ystep,xpartial)+viewy;
        xtile=focaltx+xtilestep;
        xspot=(word)((xtile<<mapshift)+((uint32_t)yintercept>>16));
        xintercept=FixedMul(xstep,ypartial)+viewx;
        ytile=focalty+ytilestep;
        yspot=(word)((((uint32_t)xintercept>>16)<<mapshift)+ytile);
        texdelta=0;

        // Special treatment when player is in back tile of pushwall
        if(playerInPushwallBackTile)
        {
#ifdef ALUMULTIPUSH
            pw = GetCurPush(focaltx, focalty);
#endif
            if(    pwalldir == di_east && xtilestep ==  1
                || pwalldir == di_west && xtilestep == -1)
            {
                int32_t yintbuf = yintercept - ((ystep * (64 - pwallpos)) >> 6);
                if((yintbuf >> 16) == focalty)   // ray hits pushwall back?
                {
                    if(pwalldir == di_east)
                        xintercept = (focaltx << TILESHIFT) + (pwallpos << 10);
                    else
                        xintercept = (focaltx << TILESHIFT) - TILEGLOBAL + ((64 - pwallpos) << 10);
                    yintercept = yintbuf;
                    tilehit = pwalltile;
                    HitVertWall();
                    continue;
                }
            }
            else if(pwalldir == di_south && ytilestep ==  1
                ||  pwalldir == di_north && ytilestep == -1)
            {
                int32_t xintbuf = xintercept - ((xstep * (64 - pwallpos)) >> 6);
                if((xintbuf >> 16) == focaltx)   // ray hits pushwall back?
                {
                    xintercept = xintbuf;
                    if(pwalldir == di_south)
                        yintercept = (focalty << TILESHIFT) + (pwallpos << 10);
                    else
                        yintercept = (focalty << TILESHIFT) - TILEGLOBAL + ((64 - pwallpos) << 10);
                    tilehit = pwalltile;
                    HitHorizWall();
                    continue;
                }
            }
        }

        do
        {
            if(ytilestep==-1 && (yintercept>>16)<=ytile) goto horizentry;
            if(ytilestep==1 && (yintercept>>16)>=ytile) goto horizentry;
vertentry:
            if((uint32_t)yintercept>mapheight*65536-1 || (word)xtile>=mapwidth)
            {
                if(xtile<0) xintercept=0;
                if(xtile>=mapwidth) xintercept=mapwidth<<TILESHIFT;
                if(yintercept<0) yintercept=0;
                if(yintercept>=(mapheight<<TILESHIFT)) yintercept=mapheight<<TILESHIFT;
                yspot=0xffff;
                tilehit=0;
                HitHorizBorder();
                break;
            }
            if(xspot>=maparea) break;
            tilehit=((byte *)tilemap)[xspot];
            if(tilehit)
            {
                if(tilehit&0x80)
                {
                    int32_t yintbuf=yintercept+(ystep>>1);
                    if((yintbuf>>16)!=(yintercept>>16))
                        goto passvert;
                    if((word)yintbuf<doorposition[tilehit&0x7f])
                        goto passvert;
                    yintercept=yintbuf;
                    xintercept=(xtile<<TILESHIFT)|0x8000;
                    HitVertDoor();
                }
                else
                {
                    if(tilehit==64)
                    {
#ifdef ALUMULTIPUSH
                        pw = GetCurPush(xspot>>mapshift, xspot&0x3F);
#endif
                        if(pwalldir==di_west || pwalldir==di_east)
                        {
	                        int32_t yintbuf;
                            int pwallposnorm;
                            int pwallposinv;
                            if(pwalldir==di_west)
                            {
                                pwallposnorm = 64-pwallpos;
                                pwallposinv = pwallpos;
                            }
                            else
                            {
                                pwallposnorm = pwallpos;
                                pwallposinv = 64-pwallpos;
                            }
                            if(pwalldir == di_east && xtile==pwallx && ((uint32_t)yintercept>>16)==pwally
                                || pwalldir == di_west && !(xtile==pwallx && ((uint32_t)yintercept>>16)==pwally))
                            {
                                yintbuf=yintercept+((ystep*pwallposnorm)>>6);
                                if((yintbuf>>16)!=(yintercept>>16))
                                    goto passvert;

                                xintercept=(xtile<<TILESHIFT)+TILEGLOBAL-(pwallposinv<<10);
                                yintercept=yintbuf;
                                tilehit=pwalltile;
                                HitVertWall();
                            }
                            else
                            {
                                yintbuf=yintercept+((ystep*pwallposinv)>>6);
                                if((yintbuf>>16)!=(yintercept>>16))
                                    goto passvert;

                                xintercept=(xtile<<TILESHIFT)-(pwallposinv<<10);
                                yintercept=yintbuf;
                                tilehit=pwalltile;
                                HitVertWall();
                            }
                        }
                        else
                        {
                            int pwallposi = pwallpos;
                            if(pwalldir==di_north) pwallposi = 64-pwallpos;
                            if(pwalldir==di_south && (word)yintercept<(pwallposi<<10)
                                || pwalldir==di_north && (word)yintercept>(pwallposi<<10))
                            {
                                if(((uint32_t)yintercept>>16)==pwally && xtile==pwallx)
                                {
                                    if(pwalldir==di_south && (int32_t)((word)yintercept)+ystep<(pwallposi<<10)
                                            || pwalldir==di_north && (int32_t)((word)yintercept)+ystep>(pwallposi<<10))
                                        goto passvert;

                                    if(pwalldir==di_south)
                                        yintercept=(yintercept&0xffff0000)+(pwallposi<<10);
                                    else
                                        yintercept=(yintercept&0xffff0000)-TILEGLOBAL+(pwallposi<<10);
                                    xintercept=xintercept-((xstep*(64-pwallpos))>>6);
                                    tilehit=pwalltile;
                                    HitHorizWall();
                                }
                                else
                                {
                                    texdelta = -(pwallposi<<10);
                                    xintercept=xtile<<TILESHIFT;
                                    tilehit=pwalltile;
                                    HitVertWall();
                                }
                            }
                            else
                            {
                                if(((uint32_t)yintercept>>16)==pwally && xtile==pwallx)
                                {
                                    texdelta = -(pwallposi<<10);
                                    xintercept=xtile<<TILESHIFT;
                                    tilehit=pwalltile;
                                    HitVertWall();
                                }
                                else
                                {
                                    if(pwalldir==di_south && (int32_t)((word)yintercept)+ystep>(pwallposi<<10)
                                            || pwalldir==di_north && (int32_t)((word)yintercept)+ystep<(pwallposi<<10))
                                        goto passvert;

                                    if(pwalldir==di_south)
                                        yintercept=(yintercept&0xffff0000)-((64-pwallpos)<<10);
                                    else
                                        yintercept=(yintercept&0xffff0000)+((64-pwallpos)<<10);
                                    xintercept=xintercept-((xstep*pwallpos)>>6);
                                    tilehit=pwalltile;
                                    HitHorizWall();
                                }
                            }
                        }
                    }
                    else
                    {
                        xintercept=xtile<<TILESHIFT;
                        HitVertWall();
                    }
                }
                break;
            }
passvert:
            *((byte *)spotvis+xspot)=1;
            xtile+=xtilestep;
            yintercept+=ystep;
            xspot=(word)((xtile<<mapshift)+((uint32_t)yintercept>>16));
        }
        while(1);
        continue;

        do
        {
            if(xtilestep==-1 && (xintercept>>16)<=xtile) goto vertentry;
            if(xtilestep==1 && (xintercept>>16)>=xtile) goto vertentry;
horizentry:
            if((uint32_t)xintercept>mapwidth*65536-1 || (word)ytile>=mapheight)
            {
                if(ytile<0) yintercept=0;
                if(ytile>=mapheight) yintercept=mapheight<<TILESHIFT;
                if(xintercept<0) xintercept=0;
                if(xintercept>=(mapwidth<<TILESHIFT)) xintercept=mapwidth<<TILESHIFT;
                xspot=0xffff;
                tilehit=0;
                HitVertBorder();
                break;
            }
            if(yspot>=maparea) break;
            tilehit=((byte *)tilemap)[yspot];
            if(tilehit)
            {
                if(tilehit&0x80)
                {
                    int32_t xintbuf=xintercept+(xstep>>1);
                    if((xintbuf>>16)!=(xintercept>>16))
                        goto passhoriz;
                    if((word)xintbuf<doorposition[tilehit&0x7f])
                        goto passhoriz;
                    xintercept=xintbuf;
                    yintercept=(ytile<<TILESHIFT)+0x8000;
                    HitHorizDoor();
                }
                else
                {
                    if(tilehit==64)
                    {
#ifdef ALUMULTIPUSH
                        pw = GetCurPush(yspot>>mapshift, yspot&0x3F);
#endif
                        if(pwalldir==di_north || pwalldir==di_south)
                        {
                            int32_t xintbuf;
                            int pwallposnorm;
                            int pwallposinv;
                            if(pwalldir==di_north)
                            {
                                pwallposnorm = 64-pwallpos;
                                pwallposinv = pwallpos;
                            }
                            else
                            {
                                pwallposnorm = pwallpos;
                                pwallposinv = 64-pwallpos;
                            }
                            if(pwalldir == di_south && ytile==pwally && ((uint32_t)xintercept>>16)==pwallx
                                || pwalldir == di_north && !(ytile==pwally && ((uint32_t)xintercept>>16)==pwallx))
                            {
                                xintbuf=xintercept+((xstep*pwallposnorm)>>6);
                                if((xintbuf>>16)!=(xintercept>>16))
                                    goto passhoriz;

                                yintercept=(ytile<<TILESHIFT)+TILEGLOBAL-(pwallposinv<<10);
                                xintercept=xintbuf;
                                tilehit=pwalltile;
                                HitHorizWall();
                            }
                            else
                            {
                                xintbuf=xintercept+((xstep*pwallposinv)>>6);
                                if((xintbuf>>16)!=(xintercept>>16))
                                    goto passhoriz;

                                yintercept=(ytile<<TILESHIFT)-(pwallposinv<<10);
                                xintercept=xintbuf;
                                tilehit=pwalltile;
                                HitHorizWall();
                            }
                        }
                        else
                        {
                            int pwallposi = pwallpos;
                            if(pwalldir==di_west) pwallposi = 64-pwallpos;
                            if(pwalldir==di_east && (word)xintercept<(pwallposi<<10)
                                    || pwalldir==di_west && (word)xintercept>(pwallposi<<10))
                            {
                                if(((uint32_t)xintercept>>16)==pwallx && ytile==pwally)
                                {
                                    if(pwalldir==di_east && (int32_t)((word)xintercept)+xstep<(pwallposi<<10)
                                            || pwalldir==di_west && (int32_t)((word)xintercept)+xstep>(pwallposi<<10))
                                        goto passhoriz;

                                    if(pwalldir==di_east)
                                        xintercept=(xintercept&0xffff0000)+(pwallposi<<10);
                                    else
                                        xintercept=(xintercept&0xffff0000)-TILEGLOBAL+(pwallposi<<10);
                                    yintercept=yintercept-((ystep*(64-pwallpos))>>6);
                                    tilehit=pwalltile;
                                    HitVertWall();
                                }
                                else
                                {
                                    texdelta = -(pwallposi<<10);
                                    yintercept=ytile<<TILESHIFT;
                                    tilehit=pwalltile;
                                    HitHorizWall();
                                }
                            }
                            else
                            {
                                if(((uint32_t)xintercept>>16)==pwallx && ytile==pwally)
                                {
                                    texdelta = -(pwallposi<<10);
                                    yintercept=ytile<<TILESHIFT;
                                    tilehit=pwalltile;
                                    HitHorizWall();
                                }
                                else
                                {
                                    if(pwalldir==di_east && (int32_t)((word)xintercept)+xstep>(pwallposi<<10)
                                            || pwalldir==di_west && (int32_t)((word)xintercept)+xstep<(pwallposi<<10))
                                        goto passhoriz;

                                    if(pwalldir==di_east)
                                        xintercept=(xintercept&0xffff0000)-((64-pwallpos)<<10);
                                    else
                                        xintercept=(xintercept&0xffff0000)+((64-pwallpos)<<10);
                                    yintercept=yintercept-((ystep*pwallpos)>>6);
                                    tilehit=pwalltile;
                                    HitVertWall();
                                }
                            }
                        }
                    }
                    else
                    {
                        yintercept=ytile<<TILESHIFT;
                        HitHorizWall();
                    }
                }
                break;
            }
passhoriz:
            *((byte *)spotvis+yspot)=1;
            ytile+=ytilestep;
            xintercept+=xstep;
            yspot=(word)((((uint32_t)xintercept>>16)<<mapshift)+ytile);
        }
        while(1);
    }
}

/*
====================
=
= WallRefresh
=
====================
*/

void WallRefresh (void)
{
    xpartialdown = viewx&(TILEGLOBAL-1);
    xpartialup = TILEGLOBAL-xpartialdown;
    ypartialdown = viewy&(TILEGLOBAL-1);
    ypartialup = TILEGLOBAL-ypartialdown;
	pw = 0;

    min_wallheight = viewheight;
#ifdef ALULIGHTING
    outsidepost = false;
#endif
    lastside = -1;                  // the first pixel is on a new wall
    AsmRefresh ();
    ScalePost ();                   // no more optimization on last post
}

void CalcViewVariables()
{
    viewangle = GetShockedAngle();
    midangle = viewangle*(FINEANGLES/ANGLES);
    viewsin = sintable[viewangle];
    viewcos = costable[viewangle];
    if ( focallength < FOCALTHRESHOLD ) {
      viewx = player->x - FixedMul( focallength, viewcos );
      viewy = player->y + FixedMul( focallength, viewsin );
   } else {
      viewx = player->x - FixedMul( FOCALTHRESHOLD, viewcos );
      viewy = player->y + FixedMul( FOCALTHRESHOLD, viewsin );
   }

    focaltx = (short)(viewx>>TILESHIFT);
    focalty = (short)(viewy>>TILESHIFT);

    viewtx = (short)(player->x >> TILESHIFT);
    viewty = (short)(player->y >> TILESHIFT);
}

//==========================================================================

/*
========================
=
= ThreeDRefresh
=
========================
*/

void DrawSniperCross(void)
{
	CA_CacheGrChunk(SCOPEPIC);
	VWB_DrawPic(0, 0+((screenHeight/scaleFactor)-200)/2, SCOPEPIC);
	if (screenHeight%200)
	{
		VWB_Bar(0, 0, screenWidth/scaleFactor, ((screenHeight/scaleFactor)-200)/2, 0);
		VWB_Bar(0, ((screenHeight/scaleFactor)-200)/2 + 200, screenWidth/scaleFactor, ((screenHeight/scaleFactor)-200)/2, 0);
	}
	UNCACHEGRCHUNK(SCOPEPIC);
}

void DrawCrosshair(void)
{
	if (gamestate.curzoom)
		return;

	int cx = (viewwidth/2)/scaleFactor;
	int cy = (viewheight/2)/scaleFactor;

	VWB_Bar(cx - 8, cy, 5, 1, 182);
	VWB_Bar(cx + 4, cy, 5, 1, 182);
	VWB_Bar(cx, cy + 3, 1, 5, 182);
	VWB_Bar(cx, cy - 7, 1, 5, 182);
}

bool automap[MAPSIZE][MAPSIZE];
#define AUTOSCALE 1
#define FULLSCALE 2
#define AUTORANGE 20

void DrawAutomap(void)
{
	int x1 = player->tilex - AUTORANGE, x2 = player->tilex + AUTORANGE;
	int y1 = player->tiley - AUTORANGE, y2 = player->tiley + AUTORANGE;
	int sx = 320 - (AUTORANGE+1)*AUTOSCALE, sy = AUTORANGE*AUTOSCALE;
	int dx, dy;

	for (int x = x1; x <= x2; x++)
		for (int y = y1; y <= y2; y++)
		{
			dx = (x - player->tilex) * AUTOSCALE;
			dy = (y - player->tiley) * AUTOSCALE;
			if (player->tilex == x && player->tiley == y)
				VWB_Bar(sx + dx, sy + dy, AUTOSCALE, AUTOSCALE, 248);
			else if (x < 0 || y < 0 || x > MAPSIZE-1 || y > MAPSIZE-1 || !automap[x][y])
				VWB_Bar(sx + dx, sy + dy, AUTOSCALE, AUTOSCALE, 183);
			else if (automap[x][y])
			{
				if (!tilemap[x][y])
					VWB_Bar(sx + dx, sy + dy, AUTOSCALE, AUTOSCALE, 190);
				else if (tilemap[x][y] >= 128)
				{
					if (!doorposition[tilemap[x][y] - 128])
						VWB_Bar(sx + dx, sy + dy, AUTOSCALE, AUTOSCALE, 115);
					else
						VWB_Bar(sx + dx, sy + dy, AUTOSCALE, AUTOSCALE, 123);
				}
				else
					VWB_Bar(sx + dx, sy + dy, AUTOSCALE, AUTOSCALE, 183);
			}
		}

	for (objtype *ob = objlist; ob; ob = ob->next)
	{
		if (ob->flags & FL_SHOOTABLE && ob->flags & FL_VISABLE && ob->flags & FL_ATTACKMODE
        && spotvis[ob->tilex][ob->tiley] && ob != player)
		{
			dx = (ob->tilex - player->tilex) * AUTOSCALE;
			dy = (ob->tiley - player->tiley) * AUTOSCALE;
			if (ABS(dx) >= (AUTORANGE+1)*AUTOSCALE
				|| ABS(dy) >= AUTORANGE*AUTOSCALE)
				continue;
			VWB_Bar(sx + dx, sy + dy, AUTOSCALE, AUTOSCALE, 175);
		}
	}
}

void DrawFullmap(void)
{
	int sx = (viewwidth/scaleFactor)/2;
	int sy = (viewheight/scaleFactor)/2;
	int dx, dy;

	for (int x = 0; x < MAPSIZE; x++)
		for (int y = 0; y < MAPSIZE; y++)
		{
			dx = (x-32) * FULLSCALE;
			dy = (y-32) * FULLSCALE;
			if (player->tilex == x && player->tiley == y)
				VWB_Bar(sx + dx, sy + dy, FULLSCALE, FULLSCALE, 248);
			else if (x < 0 || y < 0 || x > MAPSIZE-1 || y > MAPSIZE-1 || !automap[x][y])
				VWB_Bar(sx + dx, sy + dy, FULLSCALE, FULLSCALE, 183);
			else if (automap[x][y])
			{
				if (!tilemap[x][y])
					VWB_Bar(sx + dx, sy + dy, FULLSCALE, FULLSCALE, 190);
				else if (tilemap[x][y] >= 128)
				{
					if (!doorposition[tilemap[x][y] - 128])
						VWB_Bar(sx + dx, sy + dy, FULLSCALE, FULLSCALE, 115);
					else
						VWB_Bar(sx + dx, sy + dy, FULLSCALE, FULLSCALE, 123);
				}
				else
					VWB_Bar(sx + dx, sy + dy, FULLSCALE, FULLSCALE, 183);
			}
		}

	for (objtype *ob = objlist; ob; ob = ob->next)
	{
		if (ob->flags & FL_SHOOTABLE && ob->flags & FL_VISABLE && ob->flags & FL_ATTACKMODE
        && spotvis[ob->tilex][ob->tiley] && ob != player)
		{
			dx = (ob->tilex-32) * FULLSCALE;
			dy = (ob->tiley-32) * FULLSCALE;
			VWB_Bar(sx + dx, sy + dy, FULLSCALE, FULLSCALE, 175);
		}
	}
}

void    ThreeDRefresh (void)
{
//
// clear out the traced array
//
    memset(spotvis,0,maparea);
    spotvis[player->tilex][player->tiley] = 1;       // Detect all sprites over player fix

    vbuf = VL_LockSurface(screenBuffer);
    vbuf+=screenofs;
    vbufPitch = bufferPitch;

    CalcViewVariables();

//
// follow the walls from there to the right, drawing as we go
//
    VGAClearScreen ();
#if defined(USE_FEATUREFLAGS) && defined(USE_STARSKY)
    if(GetFeatureFlags() & FF_STARSKY)
        DrawStarSky(vbuf, vbufPitch);
#endif

    WallRefresh ();

#if defined(USE_FEATUREFLAGS) && defined(USE_PARALLAX)
    if(GetFeatureFlags() & FF_PARALLAXSKY)
        DrawParallax(vbuf, vbufPitch);
#endif
#if defined(USE_FEATUREFLAGS) && defined(USE_CLOUDSKY)
    if(GetFeatureFlags() & FF_CLOUDSKY)
        DrawClouds(vbuf, vbufPitch, min_wallheight);
#endif
#ifdef USE_FLOORCEILINGTEX
    DrawFloorAndCeiling(vbuf, vbufPitch, min_wallheight);
#endif

//
// draw all the scaled images
//
    DrawScaleds();                  // draw scaled stuff

#if defined(USE_FEATUREFLAGS) && defined(USE_RAIN)
    if(GetFeatureFlags() & FF_RAIN)
        DrawRain(vbuf, vbufPitch);
#endif
#if defined(USE_FEATUREFLAGS) && defined(USE_SNOW)
    if(GetFeatureFlags() & FF_SNOW)
        DrawSnow(vbuf, vbufPitch);
#endif

    DrawPlayerWeapon ();    // draw player's hands
    
    if (messagetime > 0)
   DrawMessage ();
   DeathWarning ();
#ifdef ALUCROSSHAIR
   DrawCrosshair();
#endif
   DrawGameScore ();
   DrawHealthMessage ();
   DrawArmorMessage ();
   AmmoCount ();
   DrawKeyMessage ();
   DrawKey2Message (); 
   DrawKey3Message ();
   DrawKey4Message (); 
   DrawWeaponMessage ();
#ifdef ALUAUTOMAP
   if (Keyboard[sc_M])
   {
	IN_ClearKeysDown();
	DrawFullmap();
	VW_UpdateScreen();
	IN_Ack();
	IN_ClearKeysDown();
   }
   else
	   DrawAutomap();
#endif
    if(viewsize == 21 && gamestate.weapon != -1)

    VL_UnlockSurface(screenBuffer);
    vbuf = NULL;

// Update the automap
	for (int x = 0; x < MAPSIZE; x++)
		for (int y = 0; y < MAPSIZE; y++)
			if (spotvis[x][y])
				automap[x][y] = true;

//
// show screen and time last cycle
//

    if (fizzlein)
    {
        FizzleFade(screenBuffer, screen, 0, 0,
            screenWidth, screenHeight, 20, false);
        fizzlein = false;

        lasttimecount = GetTimeCount();          // don't make a big tic count
    }
    else
    {
#ifndef REMDEBUG
        if (fpscounter)
        {
            fontnumber = 0;
            SETFONTCOLOR(4,127);
            PrintX=4; PrintY=1;
            VWB_Bar(0,0,50,10,bordercol);
            US_PrintSigned(fps);
            US_Print(" fps");
        }
#endif
        SDL_BlitSurface(screenBuffer, NULL, screen, NULL);
        SDL_UpdateRect(screen, 0, 0, 0, 0);
    }

#ifndef REMDEBUG
    if (fpscounter)
    {
        fps_frames++;
        fps_time+=tics;

        if(fps_time>15)
        {
            fps_time-=35;
            fps=fps_frames<<1;
            fps_frames=0;
        }
    }
#endif
}

/*
========================
=
= GetMessage
=
= gets ingame messages
=
========================
*/

void GetMessage (char *lastmessage)
{
   messagetime = 150; // time for message to display

   strcpy(gamestate.message, lastmessage);
}   

/*
========================
=
= DrawMessage
=
= displays ingame messages
=
========================
*/

void DrawMessage (void)
{
   messagetime-=tics;

   fontnumber = 0;
   SETFONTCOLOR(0,184);
   PrintX=9; PrintY=3;
   US_Print(gamestate.message);
   SETFONTCOLOR(184,0);
   PrintX=8; PrintY=2;
   US_Print(gamestate.message);
} 

void DrawGameScore (void)
{
	fontnumber=0;
	SETFONTCOLOR(0,184);
	PrintX=9; PrintY=15;
	US_Print("Score: ");
	US_PrintUnsigned(gamestate.score);
	SETFONTCOLOR(184,0);
	PrintX=8; PrintY=14;
	US_Print("Score: ");
	US_PrintUnsigned(gamestate.score);
}


void DeathWarning (void)
{
	fontnumber = 0;
	SETFONTCOLOR(0,184);
	PrintX=9; PrintY=27;
	US_Print("Lives: ");
	US_PrintUnsigned(gamestate.lives);
	SETFONTCOLOR(184,0);
	PrintX=8; PrintY=26;
	US_Print("Lives: ");
	US_PrintUnsigned(gamestate.lives);
}

void DrawWeaponMessage (void)
{
	fontnumber=0;
	SETFONTCOLOR(0,184);
	PrintX=9; PrintY=39;
	US_Print("Weapon: ");
	switch (gamestate.weapon)
   {
#ifdef ALUHUDNAMES
     case wp_knife:
         US_Print("Blessed Knife");
         break;
      case wp_pistol:
         US_Print("S&W Mk22");
         break;
      case wp_beretta:
         US_Print("Beretta 92F");
         break;
      case wp_lr3000:
         US_Print("LR3000");
         break;
      case wp_mp5:
         US_Print("H&K MP5");
         break;
      case wp_type79:
         US_Print("Type 79");
         break;
      case wp_bizon:
         US_Print("Bizon RL");
         break;
      case wp_machinegun:
         US_Print("RA FW900K");
         break;
      case wp_chaingun:
         US_Print("Bianchi FA-6");
         break;
      case wp_shotgun:
         US_Print("M590A1");
         break;
      case wp_l46:
         US_Print("Steyr Scout");
         break;
      case wp_m14:
         US_Print("M14");
         break;
      case wp_dshotgun:
         US_Print("Avenger");
         break;
      case wp_deagle:
         US_Print("???");
         break;
      case wp_ak47:
         US_Print("AK47-In2");
         break;
#else
     case wp_knife:
         US_Print("Blessed Knife");
         break;
      case wp_pistol:
         US_Print("S&W Mk22");
         break;
      case wp_beretta:
         US_Print("Beretta 92F");
         break;
      case wp_lr3000:
         US_Print("LR3000");
         break;
      case wp_mp5:
         US_Print("H&K MP5");
         break;
      case wp_type79:
         US_Print("Type 79");
         break;
      case wp_bizon:
         US_Print("Mini Rocket Launcher");
         break;
      case wp_machinegun:
         US_Print("Returner Arms FW900K");
         break;
      case wp_chaingun:
         US_Print("Bianchi FA-6");
         break;
      case wp_shotgun:
         US_Print("Mossberg M590A1");
         break;
      case wp_l46:
         US_Print("Steyr Scout Tactical");
         break;
      case wp_m14:
         US_Print("M14");
         break;
      case wp_dshotgun:
         US_Print("Doubled Barrled Shotgun");
         break;
      case wp_deagle:
         US_Print("???");
         break;
      case wp_ak47:
         US_Print("AK-47 Flame Rounds");
         break;
#endif
   }
	SETFONTCOLOR(184,0);
	PrintX=8; PrintY=38;
	US_Print("Weapon: ");
	switch (gamestate.weapon)
   {
#ifdef ALUHUDNAMES
     case wp_knife:
         US_Print("Blessed Knife");
         break;
      case wp_pistol:
         US_Print("S&W Mk22");
         break;
      case wp_beretta:
         US_Print("Beretta 92F");
         break;
      case wp_lr3000:
         US_Print("LR3000");
         break;
      case wp_mp5:
         US_Print("H&K MP5");
         break;
      case wp_type79:
         US_Print("Type 79");
         break;
      case wp_bizon:
         US_Print("Bizon RL");
         break;
      case wp_machinegun:
         US_Print("RA FW900K");
         break;
      case wp_chaingun:
         US_Print("Bianchi FA-6");
         break;
      case wp_shotgun:
         US_Print("M590A1");
         break;
      case wp_l46:
         US_Print("Steyr Scout");
         break;
      case wp_m14:
         US_Print("M14");
         break;
      case wp_dshotgun:
         US_Print("Avenger");
         break;
      case wp_deagle:
         US_Print("???");
         break;
      case wp_ak47:
         US_Print("AK47-In2");
         break;
#else
     case wp_knife:
         US_Print("Blessed Knife");
         break;
      case wp_pistol:
         US_Print("S&W Mk22");
         break;
      case wp_beretta:
         US_Print("Beretta 92F");
         break;
      case wp_lr3000:
         US_Print("LR3000");
         break;
      case wp_mp5:
         US_Print("H&K MP5");
         break;
      case wp_type79:
         US_Print("Type 79");
         break;
      case wp_bizon:
         US_Print("Mini Rocket Launcher");
         break;
      case wp_machinegun:
         US_Print("Returner Arms FW900K");
         break;
      case wp_chaingun:
         US_Print("Bianchi FA-6");
         break;
      case wp_shotgun:
         US_Print("Mossberg M590A1");
         break;
      case wp_l46:
         US_Print("Steyr Scout Tactical");
         break;
      case wp_m14:
         US_Print("M14");
         break;
      case wp_dshotgun:
         US_Print("Doubled Barrled Shotgun");
         break;
      case wp_deagle:
         US_Print("???");
         break;
      case wp_ak47:
         US_Print("AK-47 Flame Rounds");
         break;
#endif
   }
}

void DrawKeyMessage (void)
{
    CA_CacheGrChunk (KEY1PIC);
    CA_CacheGrChunk (NOTHINGPIC);
	if (gamestate.keys & 1)
    VWB_DrawPic (9, 51, KEY1PIC);
    else
    VWB_DrawPic (9, 51, NOTHINGPIC);
    UNCACHEGRCHUNK (KEY1PIC);
    UNCACHEGRCHUNK (NOTHINGPIC);
}

void DrawKey2Message (void)
{
    CA_CacheGrChunk (KEY2PIC);
    CA_CacheGrChunk (NOTHINGPIC);
	if (gamestate.keys & 2)
        VWB_DrawPic (9, 63, KEY2PIC);
    else
    VWB_DrawPic (9, 63, NOTHINGPIC);
    UNCACHEGRCHUNK (KEY2PIC);
    UNCACHEGRCHUNK (NOTHINGPIC);
}

void DrawKey3Message (void)
{
    CA_CacheGrChunk (KEY3PIC);
    CA_CacheGrChunk (NOTHINGPIC);
	if (gamestate.keys & 4)
    VWB_DrawPic (43, 51, KEY3PIC);
    else
    VWB_DrawPic (43, 51, NOTHINGPIC);
    UNCACHEGRCHUNK (KEY3PIC);
    UNCACHEGRCHUNK (NOTHINGPIC);
}

void DrawKey4Message (void)
{
    CA_CacheGrChunk (KEY4PIC);
    CA_CacheGrChunk (NOTHINGPIC);
	if (gamestate.keys & 8)
        VWB_DrawPic (43, 63, KEY4PIC);
    else
    VWB_DrawPic (43, 63, NOTHINGPIC);
    UNCACHEGRCHUNK (KEY4PIC);
    UNCACHEGRCHUNK (NOTHINGPIC);
}

void DrawHealthMessage (void)
{
	fontnumber=0;
	SETFONTCOLOR(0,184);
	PrintX=9; PrintY=(viewheight/scaleFactor) - 15;
	US_Print("HP: ");
	US_PrintUnsigned(gamestate.health);
	US_Print("% ");
	SETFONTCOLOR(184,0);
	PrintX=8; PrintY=(viewheight/scaleFactor) - 16;
	US_Print("HP: ");
	US_PrintUnsigned(gamestate.health);
	US_Print("% ");
}

void DrawArmorMessage (void)
{
	fontnumber=0;
	SETFONTCOLOR(0,184);
	PrintX=9; PrintY=(viewheight/scaleFactor) - 27;
	US_Print("AP: ");
	US_PrintUnsigned(gamestate.armor);
	US_Print("% ");
	SETFONTCOLOR(184,0);
	PrintX=8; PrintY=(viewheight/scaleFactor) - 28;
	US_Print("AP: ");
	US_PrintUnsigned(gamestate.armor);
	US_Print("% ");
}

void AmmoCount (void)
{
	fontnumber=0;
	SETFONTCOLOR(0,184);
	PrintX=269; PrintY=(viewheight/scaleFactor) - 15;
	switch (gamestate.weapon)
   {
     case wp_knife:
         US_Print("-");
         US_Print("/");
         US_Print("-");
         break;
      case wp_pistol:
         US_PrintUnsigned(gamestate.clip[wp_pistol]);
         US_Print("/");
         US_PrintUnsigned(gamestate.ammo[ammo_9mm]);
         break;
      case wp_beretta:
         US_PrintUnsigned(gamestate.clip[wp_beretta]);
         US_Print("/");
         US_PrintUnsigned(gamestate.ammo[ammo_9mm]);
         break;
      case wp_mp5:
         US_PrintUnsigned(gamestate.clip[wp_mp5]);
         US_Print("/");
         US_PrintUnsigned(gamestate.ammo[ammo_9mm]);
         break;
      case wp_type79:
         US_PrintUnsigned(gamestate.clip[wp_type79]);
         US_Print("/");
         US_PrintUnsigned(gamestate.ammo[ammo_9mm]);
         break;
      case wp_bizon:
         US_PrintUnsigned(gamestate.clip[wp_bizon]);
         US_Print("/");
         US_PrintUnsigned(gamestate.ammo[ammo_rocket]);
         break;
      case wp_machinegun:
         US_PrintUnsigned(gamestate.clip[wp_machinegun]);
         US_Print("/");
         US_PrintUnsigned(gamestate.ammo[ammo_30cal]);
         break;
      case wp_lr3000:
         US_PrintUnsigned(gamestate.clip[wp_lr3000]);
         US_Print("/");
         US_PrintUnsigned(gamestate.ammo[ammo_30cal]);
         break;
      case wp_chaingun:
         US_PrintUnsigned(gamestate.clip[wp_chaingun]);
         US_Print("/");
         US_PrintUnsigned(gamestate.ammo[ammo_30cal]);
         break;
      case wp_shotgun:
         US_PrintUnsigned(gamestate.clip[wp_shotgun]);
         US_Print("/");
         US_PrintUnsigned(gamestate.ammo[ammo_shells]);
         break;
      case wp_l46:
         US_PrintUnsigned(gamestate.clip[wp_l46]);
         US_Print("/");
         US_PrintUnsigned(gamestate.ammo[ammo_rifle]);
         break;
      case wp_m14:
         US_PrintUnsigned(gamestate.clip[wp_m14]);
         US_Print("/");
         US_PrintUnsigned(gamestate.ammo[ammo_rifle]);
         break;
      case wp_dshotgun:
         US_PrintUnsigned(gamestate.clip[wp_dshotgun]);
         US_Print("/");
         US_PrintUnsigned(gamestate.ammo[ammo_shells]);
         break;
      case wp_deagle:
         US_PrintUnsigned(gamestate.clip[wp_deagle]);
         US_Print("/");
         US_PrintUnsigned(gamestate.ammo[ammo_mag]);
         break;
      case wp_ak47:
         US_PrintUnsigned(gamestate.clip[wp_ak47]);
         US_Print("/");
         US_PrintUnsigned(gamestate.ammo[ammo_flame]);
         break;
   }
	SETFONTCOLOR(184,0);
	PrintX=268; PrintY=(viewheight/scaleFactor) - 16;
	switch (gamestate.weapon)
   {
     case wp_knife:
         US_Print("-");
         US_Print("/");
         US_Print("-");
         break;
      case wp_pistol:
         US_PrintUnsigned(gamestate.clip[wp_pistol]);
         US_Print("/");
         US_PrintUnsigned(gamestate.ammo[ammo_9mm]);
         break;
      case wp_beretta:
         US_PrintUnsigned(gamestate.clip[wp_beretta]);
         US_Print("/");
         US_PrintUnsigned(gamestate.ammo[ammo_9mm]);
         break;
      case wp_mp5:
         US_PrintUnsigned(gamestate.clip[wp_mp5]);
         US_Print("/");
         US_PrintUnsigned(gamestate.ammo[ammo_9mm]);
         break;
      case wp_type79:
         US_PrintUnsigned(gamestate.clip[wp_type79]);
         US_Print("/");
         US_PrintUnsigned(gamestate.ammo[ammo_9mm]);
         break;
      case wp_bizon:
         US_PrintUnsigned(gamestate.clip[wp_bizon]);
         US_Print("/");
         US_PrintUnsigned(gamestate.ammo[ammo_rocket]);
         break;
      case wp_machinegun:
         US_PrintUnsigned(gamestate.clip[wp_machinegun]);
         US_Print("/");
         US_PrintUnsigned(gamestate.ammo[ammo_30cal]);
         break;
      case wp_lr3000:
         US_PrintUnsigned(gamestate.clip[wp_lr3000]);
         US_Print("/");
         US_PrintUnsigned(gamestate.ammo[ammo_30cal]);
         break;
      case wp_chaingun:
         US_PrintUnsigned(gamestate.clip[wp_chaingun]);
         US_Print("/");
         US_PrintUnsigned(gamestate.ammo[ammo_30cal]);
         break;
      case wp_shotgun:
         US_PrintUnsigned(gamestate.clip[wp_shotgun]);
         US_Print("/");
         US_PrintUnsigned(gamestate.ammo[ammo_shells]);
         break;
      case wp_l46:
         US_PrintUnsigned(gamestate.clip[wp_l46]);
         US_Print("/");
         US_PrintUnsigned(gamestate.ammo[ammo_rifle]);
         break;
      case wp_m14:
         US_PrintUnsigned(gamestate.clip[wp_m14]);
         US_Print("/");
         US_PrintUnsigned(gamestate.ammo[ammo_rifle]);
         break;
      case wp_dshotgun:
         US_PrintUnsigned(gamestate.clip[wp_dshotgun]);
         US_Print("/");
         US_PrintUnsigned(gamestate.ammo[ammo_shells]);
         break;
      case wp_deagle:
         US_PrintUnsigned(gamestate.clip[wp_deagle]);
         US_Print("/");
         US_PrintUnsigned(gamestate.ammo[ammo_mag]);
         break;
      case wp_ak47:
         US_PrintUnsigned(gamestate.clip[wp_ak47]);
         US_Print("/");
         US_PrintUnsigned(gamestate.ammo[ammo_flame]);
         break;
   }
}

#ifdef USE_FLOORCEILINGTEX
// Textured Floor and Ceiling by DarkOne
// With multi-textured floors and ceilings stored in lower and upper bytes of
// according tile in third mapplane, respectively.
void DrawFloorAndCeiling(byte *vbuf, unsigned vbufPitch, int min_wallheight)
{
    fixed dist;                                // distance to row projection
    fixed tex_step;                            // global step per one screen pixel
    fixed gu, gv, du, dv;                      // global texture coordinates
    int u, v;                                  // local texture coordinates
    byte *toptex, *bottex;
    unsigned lasttoptex = 0xffffffff, lastbottex = 0xffffffff;

    int halfheight = viewheight >> 1;
    int y0 = min_wallheight >> 3;              // starting y value
    if(y0 > halfheight)
        return;                                // view obscured by walls
    if(!y0) y0 = 1;                            // don't let division by zero
    unsigned bot_offset0 = vbufPitch * (halfheight + y0);
    unsigned top_offset0 = vbufPitch * (halfheight - y0 - 1);

    // draw horizontal lines
    for(int y = y0, bot_offset = bot_offset0, top_offset = top_offset0;
        y < halfheight; y++, bot_offset += vbufPitch, top_offset -= vbufPitch)
    {
        dist = (heightnumerator / (y + 1)) << 5;
        gu =  viewx + FixedMul(dist, viewcos);
        gv = -viewy + FixedMul(dist, viewsin);
        tex_step = int32_t((dist << 8) / viewwidth / (175 + totalExcursion / (1113.6 / 4.25)));
        du =  FixedMul(tex_step, viewsin);
        dv = -FixedMul(tex_step, viewcos);
        gu -= (viewwidth >> 1) * du;
        gv -= (viewwidth >> 1) * dv; // starting point (leftmost)
#ifdef USE_SHADING
#ifdef ALULIGHTING
		byte *curshades;
#else
        byte *curshades = shadetable[GetShade(y << 3)];
#endif
#endif
        for(int x = 0, bot_add = bot_offset, top_add = top_offset;
            x < viewwidth; x++, bot_add++, top_add++)
        {
            if(wallheight[x] >> 3 <= y)
            {
                int curx = (gu >> TILESHIFT) & (MAPSIZE - 1);
                int cury = (-(gv >> TILESHIFT) - 1) & (MAPSIZE - 1);
                unsigned curtex = MAPSPOT(curx, cury, 2);
                if(curtex)
                {
                    unsigned curtoptex = curtex >> 8;
                    if (curtoptex != lasttoptex)
                    {
                        lasttoptex = curtoptex;
                        toptex = PM_GetTexture(curtoptex);
                    }
                    unsigned curbottex = curtex & 0xff;
                    if (curbottex==151)
                       curbottex+= (frameon>>3)&5; 
                       if (curbottex==161)
                       curbottex+= (frameon>>3)&7; 
                       if (curbottex==203)
                       curbottex+= (frameon>>5)&3;
                       if (curbottex==207)
                       curbottex+= (frameon>>3)&7; 
                    if (curbottex != lastbottex)
                    {
                        lastbottex = curbottex;
                        bottex = PM_GetTexture(curbottex);
                    }
                    u = (gu >> (TILESHIFT - TEXTURESHIFT)) & (TEXTURESIZE - 1);
                    v = (gv >> (TILESHIFT - TEXTURESHIFT)) & (TEXTURESIZE - 1);
                    unsigned texoffs = (u << TEXTURESHIFT) + (TEXTURESIZE - 1) - v;
#ifdef ALULIGHTING
                    bool shademode = 0;     // 0 = normal, 1 = indoor
                    if (tilemap[curx][cury] >= 128)
                    {
                        if (doorobjlist[tilemap[curx][cury]-128].vertical)
                        {
                            if (u >= TEXTURESIZE / 2)
                                shademode = !(TILEISOUTSIDE(curx + 1, cury));
                            else
                                shademode = !(TILEISOUTSIDE(curx - 1, cury));
                        }
                        else
                        {
                            if (v >= TEXTURESIZE / 2)
                                shademode = !(TILEISOUTSIDE(curx, cury - 1));
                            else
                                shademode = !(TILEISOUTSIDE(curx, cury + 1));
                        }
                    }
                    else
					{
                        if (curtoptex)
						{
							if (TILEISOUTSIDE(curx, cury))
								shademode = false;
							else
	                            shademode = true;
						}
					}

                    if (shademode)
                        curshades = indoortable[GetShade(y << 3, 1)];
                    else
                        curshades = shadetable[GetShade(y << 3, 0)];
#else
                        curshades = shadetable[GetShade(y << 3)];
#endif


#ifdef USE_SHADING
                    if(curtoptex && toptex[texoffs] != 255)
                        vbuf[top_add] = curshades[toptex[texoffs]];
                    if(curbottex && bottex[texoffs] != 255)
                        vbuf[bot_add] = curshades[bottex[texoffs]];
#else
                    if(curtoptex && toptex[texoffs] != 255)
                        vbuf[top_add] = toptex[texoffs];
                    if(curbottex && bottex[texoffs] != 255)
                        vbuf[bot_add] = bottex[texoffs];
#endif
                }
            }
            gu += du;
            gv += dv;
        }
    }
}
#endif
